package org.jastadd.ragconnect.compiler;

import beaver.Parser;
import org.jastadd.ragconnect.ast.*;
import org.jastadd.ragconnect.parser.RagConnectParser;
import org.jastadd.ragconnect.scanner.RagConnectScanner;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * Testing Ros2Rag without parser.
 *
 * @author rschoene - Initial contribution
 */
public class SimpleMain {

  // --- just testing byte[] conversion ---
  public static void testing() {
    System.out.println("---");
    try {
      Class<?> clazz = Class.forName("java.util.List");
      System.out.println("clazz.getName() = " + clazz.getName());
      System.out.println(Integer.class.isAssignableFrom(Integer.class));
      System.out.println(new SimpleJavaTypeUse("int").assignableTo(new SimpleJavaTypeUse("Integer")));
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
    System.out.println("---");

    byte[] bytes;
    int i = 1;
    double d = 2.3d;
    float f = 4.2f;
    short sh = 13;
    long l = 7L;
    String s = "Hello";
    char c = 'a';

    Integer ii = Integer.valueOf(1);
    if (!ii.equals(i)) throw new AssertionError("Ints not equal");

    // int to byte
    ByteBuffer i2b = ByteBuffer.allocate(4);
    i2b.putInt(i);
    bytes = i2b.array();

    // byte to int
    ByteBuffer b2i = ByteBuffer.wrap(bytes);
    int actual_i = b2i.getInt();
    if (i != actual_i) throw new AssertionError("Ints not equal");

    // double to byte
    ByteBuffer d2b = ByteBuffer.allocate(8);
    d2b.putDouble(d);
    bytes = d2b.array();

    // byte to double
    ByteBuffer b2d = ByteBuffer.wrap(bytes);
    double actual_d = b2d.getDouble();
    if (d != actual_d) throw new AssertionError("Doubles not equal");

    // float to byte
    ByteBuffer f2b = ByteBuffer.allocate(4);
    f2b.putFloat(f);
    bytes = f2b.array();

    // byte to float
    ByteBuffer b2f = ByteBuffer.wrap(bytes);
    float actual_f = b2f.getFloat();
    if (f != actual_f) throw new AssertionError("Floats not equal");

    // short to byte
    ByteBuffer sh2b = ByteBuffer.allocate(2);
    sh2b.putShort(sh);
    bytes = sh2b.array();

    // byte to short
    ByteBuffer b2sh = ByteBuffer.wrap(bytes);
    short actual_sh = b2sh.getShort();
    if (sh != actual_sh) throw new AssertionError("Shorts not equal");

    // long to byte
    ByteBuffer l2b = ByteBuffer.allocate(8);
    l2b.putLong(l);
    bytes = l2b.array();

    // byte to long
    ByteBuffer b2l = ByteBuffer.wrap(bytes);
    long actual_l = b2l.getLong();
    if (l != actual_l) throw new AssertionError("Longs not equal");

    // String to byte
    bytes = s.getBytes();

    // byte to String
    String actual_s = new String(bytes);
    if (!s.equals(actual_s)) throw new AssertionError("Strings not equal");

    // char to byte
    ByteBuffer c2b = ByteBuffer.allocate(2);
    c2b.putChar(c);
    bytes = c2b.array();

    // byte to char
    ByteBuffer b2c = ByteBuffer.wrap(bytes);
    char actual_c = b2c.getChar();
    if (c != actual_c) throw new AssertionError("Floats not equal");
  }

  public static void main(String[] args) {
    testing();
//    createManualAST();
  }

  private static void createManualAST() {
    RagConnect model = new RagConnect();
    Program program = parseProgram(Paths.get("ros2rag.starter","src", "main", "jastadd", "RobotModel.relast"));
    model.setProgram(program);

    MappingDefinition mappingDefinition = new MappingDefinition();
    mappingDefinition.setID("PoseToPosition");
    mappingDefinition.setFromType(makeMappingDefinitionType("int"));
    mappingDefinition.setFromVariableName("x");
    mappingDefinition.setToType(makeMappingDefinitionType("Position"));
    mappingDefinition.setContent("      pose.position.x += sqrt(.5 * size.x)\n" +
        "      MAP round(2)\n" +
        "      x = x / 100\n" +
        "      IGNORE_IF_SAME\n" +
        "    ;");
    model.addMappingDefinition(mappingDefinition);

    ReceiveFromMqttDefinition receiveFromMqttDefinition = new ReceiveFromMqttDefinition();
    receiveFromMqttDefinition.setAlwaysApply(false);
    receiveFromMqttDefinition.setToken(TokenComponent.createRef("Link.CurrentPosition"));
    receiveFromMqttDefinition.addMapping(mappingDefinition);
    model.addEndpointDefinition(receiveFromMqttDefinition);

    model.treeResolveAll();
    for (ErrorMessage error : model.errors()) {
      System.err.println(error);
    }

    System.out.println(model.generateAspect("Model"));
  }

  private static MappingDefinitionType makeMappingDefinitionType(String type) {
    JavaMappingDefinitionType result = new JavaMappingDefinitionType();
    result.setType(new SimpleJavaTypeUse(type));
    return result;
  }

  private static Program parseProgram(Path path) {
    try (BufferedReader reader = Files.newBufferedReader(path)) {
      RagConnectScanner scanner = new RagConnectScanner(reader);
      RagConnectParser parser = new RagConnectParser();
      GrammarFile grammarFile = (GrammarFile) parser.parse(scanner);
      Program program = new Program();
      program.addGrammarFile(grammarFile);
      return program;
    } catch (IOException | Parser.Exception e) {
      e.printStackTrace();
    }
    return null;
  }
}
